'use strict';Object.defineProperty(exports,'__esModule',{value:true});var chessground=require('chessground'),chessops=require('@publishvue/chessops'),VueCustomTooltip=require('@adamdehaven/vue-custom-tooltip');function _interopDefaultLegacy(e){return e&&typeof e==='object'&&'default'in e?e:{'default':e}}var VueCustomTooltip__default=/*#__PURE__*/_interopDefaultLegacy(VueCustomTooltip);function simpleFetch(url, params, callback) {
    params.headers = params.headers || {};
    if (params.asForm)
      { params.headers["Content-Type"] = "application/x-www-form-urlencoded"; }
    if (params.asJson) { params.headers.Accept = "application/json"; }
    if (params.asVndLichessV3Json) {
      params.headers.Accept = "application/vnd.lichess.v3+json";
      params.asJson = true;
    }
    if (params.asNdjson) { params.headers.Accept = "application/x-ndjson"; }
    if (params.accessToken)
      { params.headers.Authorization = "Bearer " + params.accessToken; }
    if (params.server)
      { api(
        "request:fetch",
        {
          url: url,
          params: params
        },
        function (result) { return callback(result); }
      ); }
    else
      { fetch(url, params).then(
        function (response) { return response.text().then(
            function (text) {
              if (params.asJson || params.asNdjson) {
                try {
                  var obj;
                  if (params.asNdjson) {
                    obj = text
                      .split("\n")
                      .filter(function (line) { return line.length; })
                      .map(function (line) { return JSON.parse(line); });
                  } else {
                    obj = JSON.parse(text);
                  }
                  try {
                    callback({ ok: true, content: obj });
                  } catch (err) {
                    console.log(err, obj);
                  }
                } catch (err) {
                  console.log("fetch parse json error", err);
                  callback({ ok: false, status: "Error: Could not parse json." });
                }
              } else {
                callback({ ok: true, content: text });
              }
            },
            function (err) {
              console.log("fetch get response text error", err);
              callback({
                ok: false,
                status: "Error: Failed to get response text."
              });
            }
          ); },
        function (err) {
          console.log("fetch error", err);
          callback({ ok: false, status: "Error: Failed to fetch." });
        }
      ); }
  }

  function getLocal(key, deffault){
    var stored = localStorage.getItem(key);
    if(stored === null) { return deffault }
    try{
      var blob = JSON.parse(stored);
      return blob
    }catch(err){
      return deffault
    }
  }

  function setLocal(key, value){
    localStorage.setItem(key, JSON.stringify(value, null, 2));
  }

var BOOK_MOVE_RATINGS = {
  0: {class: "unrated"},
  1: {class: "forcedloss"},
  2: {class: "losing"},
  3: {class: "troll"},
  4: {class: "dubious"},
  5: {class: "experimental"},
  6: {class: "stable"},
  7: {class: "promising"},
  8: {class: "good"},
  9: {class: "winning"},
  10: {class: "forcedwin"},
};

var BookMove = function BookMove(san){
  this.blob = {
    san: san,
    rating: 0
  };
};

var prototypeAccessors = { rating: { configurable: true },class: { configurable: true } };

BookMove.prototype.serialize = function serialize (){
  return this.blob
};

BookMove.prototype.deserialize = function deserialize (blob){
  this.blob = {
    san: blob.san,
    rating: blob.rating || 0
  };

  return this
};

prototypeAccessors.rating.get = function (){
  return this.blob.rating
};

prototypeAccessors.rating.set = function (rating){
  this.blob.rating = rating;
};

prototypeAccessors.class.get = function (){
  return BOOK_MOVE_RATINGS[this.rating].class
};

Object.defineProperties( BookMove.prototype, prototypeAccessors );

var BookPosition = function BookPosition(variant, fen){
  this.blob = {
    variant: variant,
    fen: fen,
    moves: {}
  };

  this.fromStored();
};

var prototypeAccessors$1 = { variant: { configurable: true },fen: { configurable: true },storeKey: { configurable: true },moves: { configurable: true } };

prototypeAccessors$1.variant.get = function (){
  return this.blob.variant
};

prototypeAccessors$1.variant.set = function (variant){
  this.blob.variant = variant;
};

prototypeAccessors$1.fen.get = function (){
  return this.blob.fen
};

prototypeAccessors$1.fen.set = function (fen){
  this.blob.fen = fen;
};

prototypeAccessors$1.storeKey.get = function (){
  return ("bookposition/" + (this.variant) + "/" + (this.fen))
};

BookPosition.prototype.getStoredBlob = function getStoredBlob (){
  return getLocal(this.storeKey, this.blob)
};

BookPosition.prototype.storeBlob = function storeBlob (){
  setLocal(this.storeKey, this.serialize());

  return this
};

BookPosition.prototype.fromStored = function fromStored (){
  var blob = this.getStoredBlob();

  return this.deserialize(blob)
};

prototypeAccessors$1.moves.get = function (){
  return this.blob.moves
};

prototypeAccessors$1.moves.set = function (moves){
  this.blob.moves = moves;
};

BookPosition.prototype.serialize = function serialize (){
  return {
    variant: this.variant,
    fen: this.fen,
    moves: Object.fromEntries(Object.entries(this.moves).map(function (entry) { return [entry[0], entry[1].serialize()]; }))
  }
};

prototypeAccessors$1.moves.get = function (){
  return this.blob.moves
};

BookPosition.prototype.getMove = function getMove (san){
  var move = this.moves[san] || new BookMove(san);
  this.moves[san] = move;
  return move
};

BookPosition.prototype.getRating = function getRating (san){
  return this.getMove(san).rating
};

BookPosition.prototype.setRating = function setRating (san, rating){
  this.getMove(san).rating = rating;
};

BookPosition.prototype.getClass = function getClass (san){
  return this.getMove(san).class
};

BookPosition.prototype.deserialize = function deserialize (blob){
  this.variant = blob.variant;
  this.fen = blob.fen;
  this.moves = Object.fromEntries(Object.entries(blob.moves).map(function (entry) { return [entry[0], new BookMove().deserialize(entry[1])]; }));

  return this
};

Object.defineProperties( BookPosition.prototype, prototypeAccessors$1 );var LICHESS_BOOK_URL = "https://explorer.lichess.ovh/lichess";

var LICHESS_BOOK_MAX_MOVES = 12;
var LICHESS_BOOK_AVG_RATINGS = [1600, 1800, 2000, 2200, 2500];
var LICHESS_BOOK_TIME_CONTROLS = ["bullet", "blitz", "rapid", "classical"];

var STANDARD_START_FEN =
  "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";

var DEFAULT_VARIANT = "standard";

var bookCache = {};

function requestLichessBook(
  fenOpt,
  variantOpt,
  maxMovesOpt,
  ratingListOpt,
  speedListOpt
) {
  var fen = fenOpt || STANDARD_START_FEN;
  var variant = variantOpt || DEFAULT_VARIANT;
  var maxMoves = maxMovesOpt || LICHESS_BOOK_MAX_MOVES;
  var ratingList = ratingListOpt || LICHESS_BOOK_AVG_RATINGS;
  var speedList = speedListOpt || LICHESS_BOOK_TIME_CONTROLS;

  var key = fen + "|" + variant + "|" + maxMoves + "|" + ratingList + "|" + speedList;

  if (bookCache[key]) {
    //console.log("book position found in cache for key", key);
    return Promise.resolve(bookCache[key]);
  }

  var ratings = ratingList.map(function (opt) { return ("ratings%5B%5D=" + opt); }).join("&");

  var speeds = speedList.map(function (opt) { return ("speeds%5B%5D=" + opt); }).join("&");

  var url =
    LICHESS_BOOK_URL + "?fen=" + fen + "&moves=" + maxMoves + "&variant=" + variant;

  if (ratings) { url += "&" + ratings; }

  if (speeds) { url += "&" + speeds; }

  return new Promise(function (resolve) {
    simpleFetch(
      url,
      {
        asJson: true
      },
      function (result) {
        if (result.ok) {
          result.content.fen = fen;
          bookCache[key] = result.content;
          resolve(result.content);
        }
      }
    );
  });
}

function weightedRandom(book) {
  if (!book) { return null; }

  if (!book.moves.length) { return null; }

  var sum = 0;

  for (var i$1 = 0, list = book.moves; i$1 < list.length; i$1 += 1) {
    var move = list[i$1];

    move.total = move.white + move.draws + move.black;
    sum += move.total;
  }

  var rnd = Math.floor(Math.random() * sum);

  var ctotal = book.moves[0].total;

  var i = 0;

  while (ctotal < rnd) {
    ctotal += book.moves[i++].total;
  }

  var selectedMove = book.moves[i];

  return selectedMove;
}

function makeVariant(display, chessops, explorer) {
  return {
    display: display,
    chessops: chessops,
    explorer: explorer
  };
}

var LICHESS_VARIANTS = [
  makeVariant("Standard", "chess", "standard"),
  makeVariant("Antichess", "antichess", "antichess"),
  makeVariant("Atomic", "atomic", "atomic"),
  makeVariant("Crazyhouse", "crazyhouse", "crazyhouse"),
  makeVariant("Horde", "horde", "horde"),
  makeVariant("King of the Hill", "kingofthehill", "kingOfTheHill"),
  makeVariant("Racing Kings", "racingkings", "racingKings")
];

function chessopsToExplorer(variant) {
  var search = LICHESS_VARIANTS.find(function (test) { return test.chessops === variant; });
  return search ? search.explorer : "standard";
}//

var ENTRIES_SEP = "\n--------------------\n";

var script = {
  name: "Vuechessground",  
  data: function data(){
    var variants = [
      {display: "Standard", key: "standard"},
      {display: "Atomic", key: "atomic"},
      {display: "Antichess", key: "antichess"},
      {display: "Crazyhouse", key: "crazyhouse"},
      {display: "Horde", key: "horde"},
      {display: "Racing Kings", key: "racingkings"} ];
    var selectedVariant = getLocal("board/selectedvariant", "standard");
    this.selectVariant(variants, selectedVariant);
    var pos = chessops.Pos();
    var history = getLocal("board/history", []);    
    return {      
      pos: pos,
      board: null,
      legalsans: [],
      history: history,      
      variants: variants,
      promuci: null,
      showpromotion: false,
      prompieces: [],
      prompiecesext: [],
      currentbook: null,
      bookmoves: [],
      orientation: getLocal("board/orientation", "white"),
      firstVariantChanged: true,
      randLeft: 0,
      trainOn: false,
      expectedMoves: [],
      getExpected: false,
      awaitTrainMove: false,
      trainTimeout: null,
      sankey: 0,
      settingsOpen: false,
    }
  },
  methods:{
    clearblob: function clearblob(){
      this.$refs.settingsblob.value = "";
    },
    setlocalstorage: function setlocalstorage(){
      var this$1$1 = this;

      var entries = this.$refs.settingsblob.value.split(ENTRIES_SEP);

      this.$refs.settingsblob.value = "Setting from " + (entries.length) + " entry(s) .";

      setTimeout(function (_) {
        try{
          var entriesobj = entries.map(function (entry) { return JSON.parse(entry); });

          localStorage.clear();

          entriesobj.forEach(function (item) { return localStorage.setItem(item[0], item[1]); });

          this$1$1.$refs.settingsblob.value = "Set " + (entriesobj.length) + " items, ok . Reloading.";

          setTimeout(function (_) {
            document.location.reload();
          }, 1000);
        }catch(err){
          this$1$1.$refs.settingsblob.value = "A problem occured with setting from blob. " + err;
        }
      }, 1000);
    },
    confirm: function confirm(msg){
      var resp = window.prompt(msg + " indeed ? ( y = yes )");

      if(!resp) { return false }

      return (resp.toLowerCase() === "y")
    },
    clearlocalstorage: function clearlocalstorage(){
      if(this.confirm("Clear Local storage")){
        localStorage.clear();
        this.$refs.settingsblob.value = "Local Storage cleared. Reloading.";
        setTimeout(function (_) {
          document.location.reload();
        }, 1000);
      }else {
        this.$refs.settingsblob.value = "Clearing Local Storage canceled.";
      }
    },
    copylocalstorage: function copylocalstorage(){
      var blob = Object.entries(localStorage).map(function (entry) { return JSON.stringify(entry); }).join(ENTRIES_SEP);

      this.$refs.settingsblob.value = blob;
      this.$refs.settingsblob.select();
      document.execCommand("copy");
    },
    settings: function settings(){
      this.settingsOpen = !this.settingsOpen;
    },
    sanratingstep: function sanratingstep(san, dir){
      var bookpos = new BookPosition(this.pos.pos.rules, this.pos.reportFen());

      var rating = bookpos.getRating(san) + dir;

      if(rating < 0) { rating = 10; }

      if(rating > 10) { rating = 0; }

      bookpos.setRating(san, rating);

      bookpos.storeBlob();

      this.buildLegalSans();
    },
    sanratingminus: function sanratingminus(ev){
      this.sanratingstep(ev.target.getAttribute("san"), -1);
    },
    sanratingplus: function sanratingplus(ev){
      this.sanratingstep(ev.target.getAttribute("san"), 1);
    },
    showRandLeft: function showRandLeft(){
      var this$1$1 = this;
      
      if(this.trainOn){
        setTimeout(function (_) {
          this$1$1.$refs.traininfo.innerHTML = "<span class=\"settingupopeningpos\">Setting up opening position ( left " + (this$1$1.randLeft) + " ) ...</span>";
        }, 0); 
      }      
    },
    trainmoveplayed: function trainmoveplayed(san){
      var this$1$1 = this;

      if(this.trainOn){
        var expectedIndex = this.expectedMoves.findIndex(function (item) { return item.san == san; });       

        if(expectedIndex < 0){
          this.$refs.traininfo.innerHTML = "<span class=\"trainerror\">Not among book moves !</span>";
        }else {
          this.$refs.traininfo.innerHTML = "<div class=\"trainrankdiv\"><span class=\"trainok\">Move <span class=\"trainrank\">" + san + "</span> was <span class=\"trainrank\">Rank " + (expectedIndex + 1) + ".</span> most played.</div><div><span class=\"trainok\">Among <span class=\"trainexps\">" + (this.expectedMoves.map(function (item, i) { return ((i+1) + ". " + (item.san)); }).join(" ")) + "</span> .</span></div>";
        }

        this.trainTimeout = setTimeout(function (_) {
          this$1$1.awaitTrainMove = false;
          if(this$1$1.trainOn) { this$1$1.train(); }
          this$1$1.trainTimeout = null;
        }, 5000);
      }
    },
    train: function train(){
      this.trainOn = true;
      this.rndpos();
    },
    stoptrain: function stoptrain(){
      this.trainOn = false;
      this.awaitTrainMove = false;
      this.randLeft = 0;      
      if(this.trainTimeout) { clearTimeout(this.trainTimeout); }
      this.trainTimeout = null;
    },
    rndpos: function rndpos(){            
      this.variantchanged();
      this.disable();
      this.randLeft = 3 + Math.floor(Math.random() * 10);
      this.showRandLeft();
      this.makeWeightedrandom();
    },
    disable: function disable(){
      var cont = this.$refs.vuechessground;
      var rect = cont.getBoundingClientRect();      
      var style = this.$refs.disablediv.style;
      style.width = (rect.width - 12) + "px";
      style.height = (rect.height - 12) + "px";
    },
    enable: function enable(){
      var style = this.$refs.disablediv.style;
      style.width = "0px";
      style.height = "0px";
    },
    selectVariant: function selectVariant(variants, variant){      
      variants.forEach(function (item) {
        item.selected = item.key === variant;
      });
      setLocal("board/selectedvariant", variant);
    },
    flip: function flip(){
      this.orientation = this.orientation == "white" ? "black" : "white";
      setLocal("board/orientation", this.orientation);
      this.setUp();
    },
    openanalysis: function openanalysis(){
      var explorer = chessopsToExplorer(this.pos.pos.rules);
      var url = "https://lichess.org/analysis/" + explorer + "/" + (this.pos.reportFen());
      window.open(url, "_blank");
    },
    getCurrent: function getCurrent(){
      var this$1$1 = this;

      return this.history.find(function (item) { return item.fen === this$1$1.pos.reportFen(); })
    },
    getCurrentIndex: function getCurrentIndex(){
      var this$1$1 = this;

      return this.history.findIndex(function (item) { return item.fen === this$1$1.pos.reportFen(); })
    },
    getBook: function getBook(){
      var this$1$1 = this;
       
      this.currentbook = null;
      this.bookmoves = [];
      requestLichessBook(
        this.pos.reportFen(),
        chessopsToExplorer(this.pos.pos.rules)
      ).then(function (result) {
        //console.log("book", result);
        this$1$1.currentbook = result;
        this$1$1.bookmoves = result.moves;        
        if(this$1$1.trainOn){
          if((!this$1$1.randLeft) && (!this$1$1.awaitTrainMove)){
              this$1$1.expectedMoves = this$1$1.bookmoves.map(function (bookmove) { return ({
                san: bookmove.san,
                totalPlays: bookmove.white + bookmove.draws + bookmove.black
              }); }).sort(function (a,b) { return b.totalPlays - a.totalPlays; });              
              this$1$1.$refs.traininfo.innerHTML = "<span class=\"makeyourmove\">Make your move !</span>";
              this$1$1.awaitTrainMove = true;
              this$1$1.orientation = this$1$1.getCurrent().fen.split(" ")[1] === "w" ? "white" : "black";
              this$1$1.setUp(true);
            }
        }
      });
    },
    makeWeightedrandom: function makeWeightedrandom() {
      var this$1$1 = this;

      if (this.currentbook) {
        if (this.currentbook.fen === this.pos.reportFen()) {
          var selectedMove = weightedRandom(this.currentbook);

          if (!selectedMove) {
            //console.log("no book move for the position");

            this.randLeft = 0;

            this.trainOn = false;

            this.enable();

            return;
          }

          var san = selectedMove.san;

          this.makesanmove(null, san);

          if(this.randLeft) {
            this.randLeft--;            
          }

          if(this.trainOn && this.randLeft){
            this.showRandLeft();
            setTimeout(function (_) {
              this$1$1.makeWeightedrandom();
            }, 250);
          } else {
            this.randLeft = 0;

            this.enable();
          }
        } else {
          //console.log("book out of sync with position for weighted random");

          if(this.randLeft){
            setTimeout(function (_) {
              this$1$1.makeWeightedrandom();
            }, 250);
          } else {
            this.enable();
          }
        }
      } else {
        //console.log("no book to make weighted random from");

        if(this.randLeft){
            setTimeout(function (_) {
              this$1$1.makeWeightedrandom();
            }, 250);
        } else {
          this.enable();
        }
      }
    },
    makemovecancel: function makemovecancel(){
      var this$1$1 = this;

      this.prompieces = [];
      this.prompiecesext = [];
      this.showpromotion = false;
      this.$refs.movetextinput.value = "";
      setTimeout(function (_) {
        this$1$1.setUp();
      }, 250);      
    },
    pieceToName: function pieceToName(p){
      return {
        q: "queen",
        r: "rook",
        b: "bishop",
        n: "knight",
        k: "king",
        p: "pawn"
      }[p]
    },
    pieceToStyle: function pieceToStyle(p, fen){
      var parts = fen.split(" ");
      var turn = parts[1] == "w" ? "white" : "black";
      var name = this.pieceToName(p);
      var style = "piece " + name + " " + turn;      
      return style
    },
    step: function step(dir){
      var fen = this.pos.reportFen();
      //console.log("step", dir, fen)
      var index = this.history.findIndex(function (item) { return item.fen === fen; });
      if(index < 0){
        //console.error("fen not found", fen)
        return
      }
      index += dir;
      if(index < 0) { index = 0; }
      if(index >= this.history.length) { index = this.history.length - 1; }
      this.pos.setFen(this.history[index].fen);
      this.getSelectedHistory();
      this.setUp();
      this.prompieces = [];
      this.prompiecesext = [];
    },
    tobegin: function tobegin(){
      this.step(-100);
    },
    back: function back(){
      this.step(-1);
    },
    forward: function forward(){
      this.step(1);
    },
    toend: function toend(){
      this.step(100);
    },
    del: function del(){
      this.back();
      var fen = this.pos.reportFen();
      //console.log("step", dir, fen)
      var index = this.history.findIndex(function (item) { return item.fen === fen; });
      if(index >= 0){
        this.history = this.history.slice(0, index + 1);
      }
    },
    clickhistory: function clickhistory(ev) {
      if(this.trainOn) { return window.alert("Not allowed in train mode.") }
      var fen = ev.target.getAttribute("fen");      
      this.pos.setFen(fen);
      this.getSelectedHistory();            
      var itemIndex = this.history.findIndex(function (item) { return item.selected == "histactive"; });
      //console.log(itemIndex, this.history[itemIndex])
      this.recordMovePlayed(this.history[itemIndex].genSan, this.history[itemIndex].genUci, itemIndex > 0 ? this.history[itemIndex - 1].fen : null, true);
    },
    getSelectedHistory: function getSelectedHistory(){
      var fen = this.pos.reportFen();
      for(var i = 0, list = this.history; i < list.length; i += 1) {
        var item = list[i];

        item.selected = item.fen == fen ? "histactive" : "histpassive";
      }
    },
    recordMovePlayed: function recordMovePlayed(san, uci, prevFen, dontAdd){      
      var fen = this.pos.reportFen();            

      if(!dontAdd){
        var index = this.history.findIndex(function (item) { return item.fen === prevFen; });
        
        if(index < 0){        
          //console.log("previous fen not in history")
          this.history.push({ genSan: san, genUci: uci, fen: fen });        
        }else {
          //console.log("previous in history, trimming")
          this.history = this.history.slice(0, index + 1);
          this.history.push({ genSan: san, genUci: uci, fen: fen });
        }
      }

      this.getSelectedHistory();      
      this.prompieces = [];
      this.prompiecesext = [];
      this.setUp();

      var event = {
        kind: "moveplayed",
        prevFen: prevFen,
        fen: fen,
        genSan: san,
        genUci: uci,
        variant: this.pos.pos.rules,
        history: this.history,
      };

      this.$emit("moveplayed", event);
    },
    prompiececlicked: function prompiececlicked(ev){
      var piece = ev.target.getAttribute("piece");
      var uci = this.promuci + piece;
      var san = this.pos.uciToSan(uci);
      this.$refs.movetextinput.value = san;
      this.showpromotion = false;
    },
    makesanmove: function makesanmove(ev, san){
      this.$refs.movetextinput.value = san;
      this.makemove();
    },
    makemove: function makemove(){
      var fen = this.pos.reportFen();
      var move = this.$refs.movetextinput.value;
      this.$refs.movetextinput.value = "";
      try {
        var uci = this.pos.sanToUci(move);
        this.pos.playSan(move);                
        this.recordMovePlayed(move, uci, fen);
      }catch(err){
        try{
          var san = this.pos.uciToSan(move);
          this.pos.playSan(san);
          this.recordMovePlayed(san, move, fen);          
        }catch(err){
          window.alert(("Illegal move " + move + " !"));
          return
        }
      }
    },
    variantchanged: function variantchanged(){      
      var variant = this.$refs.variantselect ? this.$refs.variantselect.value : this.pos.pos.rules;
      this.selectVariant(this.variants, variant);
      this.pos.setVariant(variant);      
      this.history = this.firstVariantChanged ? getLocal("board/history", []) : [];
      this.firstVariantChanged = false;
      if(this.history.length){
        var item = this.history.find(function (item) { return item.selected === "histactive"; });        
        this.pos.setFen(item.fen);
        this.setUp();
      } else {
        this.recordMovePlayed("*", "*", null);        
      }
    },
    reset: function reset(){
      this.variantchanged();
    },
    clicksan: function clicksan(ev){      
      var fen = this.pos.reportFen();
      var san = ev.target.getAttribute("san");      
      var uci = this.pos.sanToUci(san);
      this.pos.playSan(san);
      this.recordMovePlayed(san, uci, fen);
    },
    buildLegalSans: function buildLegalSans(){
      var this$1$1 = this;

      var bookpos = new BookPosition(this.pos.pos.rules, this.pos.reportFen());
      this.legalsans = this.pos.allLegalSans().map(function (san) {
        var nextsan = null;
        var curr = this$1$1.getCurrentIndex();
        if((curr >= 0) && (curr + 1 < this$1$1.history.length)){
          nextsan = this$1$1.history[curr + 1].genSan;
        }
        var sortvalue = 0;
        var klass = "legalsan";
        if(san === nextsan) {
          klass += " histsan";
          sortvalue = 1000;
        }                
        var rating = bookpos.getRating(san);
        sortvalue += 100 * rating;
        klass += " " + bookpos.getClass(san);        
        return ({
          san: san,
          klass: klass,
          rating: rating,
          sortvalue: sortvalue,
          key: this$1$1.sankey++
        })
      }).sort(function (a, b) { return a.san.localeCompare(b.san); }).sort(function (a,b) { return b.sortvalue - a.sortvalue; });
    },
    setUp: function setUp(skipBook){
      var fen = this.pos.reportFen();
      this.$refs.fentextinput.value = fen;
      var config = {
          fen: fen,        
          movable: { events: { after: this.movePlayed() } },
          orientation: this.orientation,
      };
      this.board = chessground.Chessground(this.$refs.board, config);      
      try{
        var lastUci = this.getCurrent().genUci;
        if(lastUci.length >= 4){
          var srcDest = [lastUci.substring(0, 2), lastUci.substring(2, 4)];
          this.board.set({lastMove: srcDest});          
        }        
      }catch(err){window.alert(("could not highlight move " + err));}
      this.buildLegalSans();
      setLocal("board/history", this.history);
      if(!skipBook) { this.getBook(); }
    },
    movePlayed: function movePlayed(){
      var this$1$1 = this;

      return function (orig, dest, metadata) {
        var fen = this$1$1.pos.reportFen();
        var uci = "" + orig + dest;        
        var legals = this$1$1.pos.legalsForUci(uci);
        var num = legals.length;
        if(!num){
          window.alert(("Illegal move " + uci + " !"));
          this$1$1.$refs.movetextinput.value = "";
          this$1$1.showpromotion = false;
          this$1$1.setUp();
          return
        }
        if(num > 1){
          // handle promotion
          this$1$1.prompieces = legals.map(function (legal) { return legal.substring(4, 5); });
          this$1$1.prompiecesext = this$1$1.prompieces.map(function (p) { return ({
            p: p,
            klass: "prompiecediv " + this$1$1.pieceToStyle(p, this$1$1.pos.reportFen())              
          }); });
          this$1$1.showpromotion = true;          
          this$1$1.$refs.movetextinput.value = "Press piece !";
          this$1$1.promuci = uci;          
          return
        }
        this$1$1.$refs.movetextinput.value = "";
        this$1$1.showpromotion = false;
        var san = this$1$1.pos.uciToSan(uci);
        this$1$1.pos.playUci(uci);        
        this$1$1.recordMovePlayed(san, uci, fen);        

        this$1$1.trainmoveplayed(san);
      }
    }
  },
  mounted: function mounted(){    
    this.variantchanged();    
  },
  components:{
    VueCustomTooltip: VueCustomTooltip__default["default"]
  }
};function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    var options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    var hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            var originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            var existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}/* script */
var __vue_script__ = script;
/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticClass:"trainvertcont"},[_vm._ssrNode("<div class=\"traincontrolsdiv\">"+((_vm.trainOn)?("<div class=\"traininfo\"></div>"):"<!---->")+" "+((!_vm.trainOn)?("<button class=\"trainbutton\">Train</button>"):"<!---->")+" "+((_vm.trainOn && !_vm.randLeft)?("<button class=\"stoptrainbutton\">Stop Training</button>"):"<!---->")+"</div> "),_vm._ssrNode("<div class=\"vuechessground\">","</div>",[_vm._ssrNode("<div id=\"disablediv\"></div> "+((_vm.settingsOpen)?("<div id=\"settingsdiv\" class=\"settingsdiv\"><button class=\"greenbutton\">Close Settings</button> <hr> <div class=\"localstoragecontrols\"><button class=\"graybutton\">Clear Blob</button> <button class=\"greenbutton\">Copy Local Storaga</button> <button class=\"yellowbutton\">Set Local Storage</button> <button class=\"redbutton\">Clear Local Storaga</button></div> <div><textarea id=\"settingsblob\"></textarea></div></div>"):"<!---->")+" "),_vm._ssrNode("<div class=\"horizcont\">","</div>",[_vm._ssrNode("<div class=\"vertcont\">","</div>",[_vm._ssrNode("<div class=\"moveinputdiv\">","</div>",[_c('VueCustomTooltip',{attrs:{"label":"flip board"}},[(!_vm.trainOn)?_c('div',{staticClass:"svgbutton spinner",on:{"click":_vm.flip}}):_vm._e()]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"label":"move text, san or uci"}},[_c('input',{ref:"movetextinput",staticClass:"movetextinput",attrs:{"type":"text"},on:{"keyup":function($event){if(!$event.type.indexOf('key')&&_vm._k($event.keyCode,"enter",13,$event.key,"Enter")){ return null; }return _vm.makemove.apply(null, arguments)}}})]),_vm._ssrNode(" "+((_vm.showpromotion)?("<div class=\"prompiecebuttons\">"+(_vm._ssrList((_vm.prompiecesext),function(prompieceext){return ("<div"+(_vm._ssrAttr("piece",prompieceext.p))+(_vm._ssrClass(null,prompieceext.klass))+">"+_vm._ssrEscape("\n              "+_vm._s(prompieceext.p.toUpperCase())+"\n            ")+"</div>")}))+"</div>"):"<!---->")+" "),_c('VueCustomTooltip',{attrs:{"label":"make text move"}},[(!_vm.trainOn && !_vm.showpromotion)?_c('div',{staticClass:"svgbutton checkmark greenbutton",on:{"click":_vm.makemove}}):_vm._e()]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"label":"cancel move"}},[_c('div',{staticClass:"svgbutton cross yellowbutton",on:{"click":_vm.makemovecancel}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"label":"reset board"}},[(!_vm.trainOn && !_vm.showpromotion)?_c('div',{staticClass:"svgbutton exclam redbutton",on:{"click":_vm.reset}}):_vm._e()])],2),_vm._ssrNode(" <div class=\"blue merida\"><div class=\"cg-board-wrap\"></div></div> <div class=\"fendiv\"><input tytpe=\"text\" class=\"fentextinput\"></div> "),(!_vm.trainOn)?_vm._ssrNode("<div class=\"controls\">","</div>",[_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"to begin"}},[_c('div',{staticClass:"svgbutton tobegin bluebutton",on:{"click":_vm.tobegin}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"back"}},[_c('div',{staticClass:"svgbutton arrowleft greenbutton",on:{"click":_vm.back}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"forward"}},[_c('div',{staticClass:"svgbutton arrowright greenbutton",on:{"click":_vm.forward}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"to end"}},[_c('div',{staticClass:"svgbutton toend bluebutton",on:{"click":_vm.toend}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"delete move"}},[_c('div',{staticClass:"svgbutton cross redbutton",on:{"click":_vm.del}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"select variant"}},[_c('select',{ref:"variantselect",on:{"change":_vm.variantchanged}},_vm._l((_vm.variants),function(variant){return _c('option',{key:variant.key,domProps:{"value":variant.key,"selected":variant.selected}},[_vm._v(_vm._s(variant.display))])}),0)]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"weighted random book move"}},[_c('div',{staticClass:"svgbutton questionmark yellowbutton",on:{"click":_vm.makeWeightedrandom}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"open lichess analysis"}},[_c('div',{staticClass:"svgbutton openfolder",on:{"click":_vm.openanalysis}})]),_vm._ssrNode(" "),_c('VueCustomTooltip',{attrs:{"position":"is-bottom","label":"settings"}},[_c('div',{staticClass:"svgbutton tool greenbutton",on:{"click":_vm.settings}})])],2):_vm._e()],2),_vm._ssrNode(" <div class=\"legalsans\">"+(_vm._ssrList((_vm.legalsans),function(item){return ("<div>"+((!_vm.trainOn)?("<div class=\"sancont\"><div"+(_vm._ssrAttr("san",item.san))+(_vm._ssrClass(null,item.klass))+">"+_vm._ssrEscape("\n              "+_vm._s(item.san)+"\n            ")+"</div> <div"+(_vm._ssrAttr("san",item.san))+" class=\"sanbutton arrowdown red\"></div> <div"+(_vm._ssrAttr("san",item.san))+" class=\"sanbutton arrowup green\"></div></div>"):"<!---->")+"</div>")}))+"</div> <div class=\"historysans\">"+(_vm._ssrList((_vm.history.slice().reverse()),function(item){return ("<div"+(_vm._ssrAttr("fen",item.fen))+(_vm._ssrClass("historysan",item.selected))+">"+_vm._ssrEscape("            \n          "+_vm._s(item.genSan)+"          \n        ")+"</div>")}))+"</div> <div class=\"bookdiv\">"+((!_vm.trainOn)?("<table cellpadding=\"2\" cellspacing=\"3\"><thead><tr><td class=\"bookmovesan\"></td> <td class=\"white\">White</td> <td class=\"draw\">Draw</td> <td class=\"black\">Black</td></tr></thead> <tbody>"+(_vm._ssrList((_vm.bookmoves),function(item){return ("<tr><td class=\"bookmovesan\">"+_vm._ssrEscape("\n                "+_vm._s(item.san)+"\n              ")+"</td> <td class=\"white\">"+_vm._ssrEscape(_vm._s(item.white))+"</td> <td class=\"draw\">"+_vm._ssrEscape(_vm._s(item.draws))+"</td> <td class=\"black\">"+_vm._ssrEscape(_vm._s(item.black))+"</td></tr>")}))+"</tbody></table>"):"<!---->")+"</div>")],2)],2)],2)};
var __vue_staticRenderFns__ = [];

  /* style */
  var __vue_inject_styles__ = undefined;
  /* scoped */
  var __vue_scope_id__ = undefined;
  /* module identifier */
  var __vue_module_identifier__ = "data-v-977df44e";
  /* functional template */
  var __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  var __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );// install function executed by Vue.use()
function install(Vue) {
  if (install.installed) { return; }
  install.installed = true;
  Vue.component('Vuechessground', __vue_component__);
}

// Create module definition for Vue.use()
var plugin = {
  install: install,
};

// To auto-install when vue is found
var GlobalVue = null;
if (typeof window !== 'undefined') {
  GlobalVue = window.Vue;
} else if (typeof global !== 'undefined') {
  GlobalVue = global.Vue;
}
if (GlobalVue) {
  GlobalVue.use(plugin);
}

// Inject install function into component - allows component
// to be registered via Vue.use() as well as Vue.component()
__vue_component__.install = install;

// It's possible to expose named exports when writing components that can
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = component;
exports["default"]=__vue_component__;//# sourceMappingURL=vuechessground.ssr.js.map
